@@ -1,5 +1,5 @@
 /* -*- c -*-
- * Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation
  *
  * Author: Nikos Mavroyanopoulos
  *
@@ -41,13 +41,13 @@ extern "C"
 {
 #endif
 
-#define LIBGNUTLS_VERSION "2.0.4"
+#define LIBGNUTLS_VERSION "2.6.2"
 
 #define LIBGNUTLS_VERSION_MAJOR 2
-#define LIBGNUTLS_VERSION_MINOR 0
-#define LIBGNUTLS_VERSION_PATCH 4
+#define LIBGNUTLS_VERSION_MINOR 6
+#define LIBGNUTLS_VERSION_PATCH 2
 
-#define LIBGNUTLS_VERSION_NUMBER 0x020004
+#define LIBGNUTLS_VERSION_NUMBER 0x020602
 
 
 
@@ -60,6 +60,7 @@ extern "C"
 #endif
 /* Get time_t. */
 #include <time.h>
+#include <gnutls/compat.h>
 
 #define GNUTLS_CIPHER_RIJNDAEL_128_CBC GNUTLS_CIPHER_AES_128_CBC
 #define GNUTLS_CIPHER_RIJNDAEL_256_CBC GNUTLS_CIPHER_AES_256_CBC
@@ -68,18 +69,34 @@ extern "C"
 
   typedef enum gnutls_cipher_algorithm
   {
+    GNUTLS_CIPHER_UNKNOWN = 0,
     GNUTLS_CIPHER_NULL = 1,
     GNUTLS_CIPHER_ARCFOUR_128,
     GNUTLS_CIPHER_3DES_CBC,
     GNUTLS_CIPHER_AES_128_CBC,
     GNUTLS_CIPHER_AES_256_CBC,
     GNUTLS_CIPHER_ARCFOUR_40,
+    GNUTLS_CIPHER_CAMELLIA_128_CBC,
+    GNUTLS_CIPHER_CAMELLIA_256_CBC,
     GNUTLS_CIPHER_RC2_40_CBC = 90,
-    GNUTLS_CIPHER_DES_CBC
+    GNUTLS_CIPHER_DES_CBC,
+
+    /* used only for PGP internals. Ignored in TLS/SSL 
+     */
+    GNUTLS_CIPHER_IDEA_PGP_CFB = 200,
+    GNUTLS_CIPHER_3DES_PGP_CFB,
+    GNUTLS_CIPHER_CAST5_PGP_CFB,
+    GNUTLS_CIPHER_BLOWFISH_PGP_CFB,
+    GNUTLS_CIPHER_SAFER_SK128_PGP_CFB,
+    GNUTLS_CIPHER_AES128_PGP_CFB,
+    GNUTLS_CIPHER_AES192_PGP_CFB,
+    GNUTLS_CIPHER_AES256_PGP_CFB,
+    GNUTLS_CIPHER_TWOFISH_PGP_CFB
   } gnutls_cipher_algorithm_t;
 
   typedef enum
   {
+    GNUTLS_KX_UNKNOWN = 0,
     GNUTLS_KX_RSA = 1,
     GNUTLS_KX_DHE_DSS,
     GNUTLS_KX_DHE_RSA,
@@ -121,6 +138,8 @@ extern "C"
     GNUTLS_MAC_SHA256,
     GNUTLS_MAC_SHA384,
     GNUTLS_MAC_SHA512
+    /* If you add anything here, make sure you align with
+       gnutls_digest_algorithm_t, in particular SHA-224. */
   } gnutls_mac_algorithm_t;
 
   /* The enumerations here should have the same value with
@@ -135,7 +154,8 @@ extern "C"
     GNUTLS_DIG_MD2 = GNUTLS_MAC_MD2,
     GNUTLS_DIG_SHA256 = GNUTLS_MAC_SHA256,
     GNUTLS_DIG_SHA384 = GNUTLS_MAC_SHA384,
-    GNUTLS_DIG_SHA512 = GNUTLS_MAC_SHA512
+    GNUTLS_DIG_SHA512 = GNUTLS_MAC_SHA512,
+    GNUTLS_DIG_SHA224
   } gnutls_digest_algorithm_t;
 
   /* exported for other gnutls headers. This is the maximum number of
@@ -146,6 +166,7 @@ extern "C"
 #define GNUTLS_COMP_ZLIB GNUTLS_COMP_DEFLATE
   typedef enum
   {
+    GNUTLS_COMP_UNKNOWN = 0,
     GNUTLS_COMP_NULL = 1,
     GNUTLS_COMP_DEFLATE,
     GNUTLS_COMP_LZO		/* only available if gnutls-extra has
@@ -194,8 +215,7 @@ extern "C"
     GNUTLS_A_UNSUPPORTED_EXTENSION = 110,
     GNUTLS_A_CERTIFICATE_UNOBTAINABLE = 111,
     GNUTLS_A_UNRECOGNIZED_NAME = 112,
-    GNUTLS_A_UNKNOWN_SRP_USERNAME = 120,
-    GNUTLS_A_MISSING_SRP_USERNAME = 121,
+    GNUTLS_A_UNKNOWN_PSK_IDENTITY = 115,
     GNUTLS_A_INNER_APPLICATION_FAILURE = 208,
     GNUTLS_A_INNER_APPLICATION_VERIFICATION = 209
   } gnutls_alert_description_t;
@@ -242,9 +262,9 @@ extern "C"
   } gnutls_certificate_request_t;
 
   typedef enum
-  { GNUTLS_OPENPGP_KEY,
-    GNUTLS_OPENPGP_KEY_FINGERPRINT
-  } gnutls_openpgp_key_status_t;
+  { GNUTLS_OPENPGP_CERT,
+    GNUTLS_OPENPGP_CERT_FINGERPRINT
+  } gnutls_openpgp_crt_status_t;
 
   typedef enum
   {
@@ -264,6 +284,7 @@ extern "C"
 
   typedef enum
   {
+    GNUTLS_CRT_UNKNOWN = 0,
     GNUTLS_CRT_X509 = 1,
     GNUTLS_CRT_OPENPGP
   } gnutls_certificate_type_t;
@@ -274,6 +295,13 @@ extern "C"
     GNUTLS_X509_FMT_PEM
   } gnutls_x509_crt_fmt_t;
 
+  typedef enum gnutls_certificate_print_formats
+    {
+      GNUTLS_CRT_PRINT_FULL,
+      GNUTLS_CRT_PRINT_ONELINE,
+      GNUTLS_CRT_PRINT_UNSIGNED_FULL
+    } gnutls_certificate_print_formats_t;
+
   typedef enum
   {
     GNUTLS_PK_UNKNOWN = 0,
@@ -295,7 +323,8 @@ extern "C"
     GNUTLS_SIGN_RSA_RMD160,
     GNUTLS_SIGN_RSA_SHA256,
     GNUTLS_SIGN_RSA_SHA384,
-    GNUTLS_SIGN_RSA_SHA512
+    GNUTLS_SIGN_RSA_SHA512,
+    GNUTLS_SIGN_RSA_SHA224
   } gnutls_sign_algorithm_t;
 
   const char *gnutls_sign_algorithm_get_name (gnutls_sign_algorithm_t
@@ -312,8 +341,12 @@ extern "C"
   struct gnutls_dh_params_int;
   typedef struct gnutls_dh_params_int *gnutls_dh_params_t;
 
-  struct gnutls_x509_privkey_int;	/* XXX ugly. */
-  typedef struct gnutls_x509_privkey_int *gnutls_rsa_params_t;	/* XXX ugly. */
+  /* XXX ugly. */
+  struct gnutls_x509_privkey_int;
+  typedef struct gnutls_x509_privkey_int *gnutls_rsa_params_t;
+
+  struct gnutls_priority_st;
+  typedef struct gnutls_priority_st *gnutls_priority_t;
 
   typedef struct
   {
@@ -365,9 +398,7 @@ extern "C"
 							 session);
 
   size_t gnutls_cipher_get_key_size (gnutls_cipher_algorithm_t algorithm);
-
-  void gnutls_session_enable_compatibility_mode (gnutls_session_t session);
-  void gnutls_record_disable_padding (gnutls_session_t session);
+  size_t gnutls_mac_get_key_size (gnutls_mac_algorithm_t algorithm);
 
 /* the name of the specified algorithms */
   const char *gnutls_cipher_get_name (gnutls_cipher_algorithm_t algorithm);
@@ -377,6 +408,17 @@ extern "C"
   const char *gnutls_kx_get_name (gnutls_kx_algorithm_t algorithm);
   const char *gnutls_certificate_type_get_name (gnutls_certificate_type_t
 						type);
+  const char *gnutls_pk_get_name (gnutls_pk_algorithm_t algorithm);
+  const char *gnutls_sign_get_name (gnutls_sign_algorithm_t algorithm);
+
+  gnutls_mac_algorithm_t gnutls_mac_get_id (const char* name);
+  gnutls_compression_method_t gnutls_compression_get_id (const char* name);
+  gnutls_cipher_algorithm_t gnutls_cipher_get_id (const char* name);
+  gnutls_kx_algorithm_t gnutls_kx_get_id (const char* name);
+  gnutls_protocol_t gnutls_protocol_get_id (const char* name);
+  gnutls_certificate_type_t gnutls_certificate_type_get_id (const char* name);
+  gnutls_pk_algorithm_t gnutls_pk_get_id (const char *name);
+  gnutls_sign_algorithm_t gnutls_sign_get_id (const char *name);
 
   /* list supported algorithms */
   const gnutls_cipher_algorithm_t *gnutls_cipher_list (void);
@@ -385,6 +427,8 @@ extern "C"
   const gnutls_protocol_t *gnutls_protocol_list (void);
   const gnutls_certificate_type_t *gnutls_certificate_type_list (void);
   const gnutls_kx_algorithm_t *gnutls_kx_list (void);
+  const gnutls_pk_algorithm_t *gnutls_pk_list (void);
+  const gnutls_sign_algorithm_t *gnutls_sign_list (void);
   const char *gnutls_cipher_suite_info (size_t i,
 					char *id,
 					gnutls_kx_algorithm_t *kx,
@@ -398,6 +442,7 @@ extern "C"
 
   void gnutls_perror (int error);
   const char *gnutls_strerror (int error);
+  const char *gnutls_strerror_name (int error);
 
 /* Semi-internal functions.
  */
@@ -417,6 +462,10 @@ extern "C"
 #define gnutls_read gnutls_record_recv
 #define gnutls_write gnutls_record_send
 
+  void gnutls_session_enable_compatibility_mode (gnutls_session_t session);
+
+  void gnutls_record_disable_padding (gnutls_session_t session);
+
   int gnutls_record_get_direction (gnutls_session_t session);
 
   size_t gnutls_record_get_max_size (gnutls_session_t session);
@@ -437,6 +486,29 @@ extern "C"
 
 /* TLS Extensions */
 
+  typedef int (*gnutls_ext_recv_func) (gnutls_session_t session,
+				       const unsigned char *data, size_t len);
+  typedef int (*gnutls_ext_send_func) (gnutls_session_t session,
+				       unsigned char *data, size_t len);
+
+  /* This flag indicates for an extension whether
+   * it is useful to application level or TLS level only.
+   * This is (only) used to parse the application level extensions
+   * before the user_hello callback is called.
+   */
+  typedef enum
+    {
+      GNUTLS_EXT_ANY,
+      GNUTLS_EXT_APPLICATION,
+      GNUTLS_EXT_TLS
+    } gnutls_ext_parse_type_t;
+
+  int gnutls_ext_register (int type,
+			   const char *name,
+			   gnutls_ext_parse_type_t parse_type,
+			   gnutls_ext_recv_func recv_func,
+			   gnutls_ext_send_func send_func);
+
   typedef enum
   {
     GNUTLS_NAME_DNS = 1
@@ -450,6 +522,26 @@ extern "C"
 			      void *data, size_t * data_length,
 			      unsigned int *type, unsigned int indx);
 
+  /* Opaque PRF Input
+   * http://tools.ietf.org/id/draft-rescorla-tls-opaque-prf-input-00.txt
+   */
+
+  void
+  gnutls_oprfi_enable_client (gnutls_session_t session,
+			      size_t len,
+			      unsigned char *data);
+
+  typedef int (*gnutls_oprfi_callback_func) (gnutls_session_t session,
+					     void *userdata,
+					     size_t oprfi_len,
+					     const unsigned char *in_oprfi,
+					     unsigned char *out_oprfi);
+
+  void
+  gnutls_oprfi_enable_server (gnutls_session_t session,
+			      gnutls_oprfi_callback_func cb,
+			      void *userdata);
+
   /* Supplemental data, RFC 4680. */
   typedef enum
     {
@@ -473,6 +565,14 @@ extern "C"
 
 /* if you just want some defaults, use the following.
  */
+  int gnutls_priority_init( gnutls_priority_t*, const char *priority, const char** err_pos);
+  void gnutls_priority_deinit( gnutls_priority_t);
+  
+  int gnutls_priority_set(gnutls_session_t session, gnutls_priority_t);
+  int gnutls_priority_set_direct(gnutls_session_t session, const char *priority, const char** err_pos);
+
+  /* for compatibility
+   */
   int gnutls_set_default_priority (gnutls_session_t session);
   int gnutls_set_default_export_priority (gnutls_session_t session);
 
@@ -508,12 +608,21 @@ extern "C"
 /* returns security values. 
  * Do not use them unless you know what you're doing.
  */
-#define TLS_MASTER_SIZE 48
-#define TLS_RANDOM_SIZE 32
+#define GNUTLS_MASTER_SIZE 48
+#define GNUTLS_RANDOM_SIZE 32
   const void *gnutls_session_get_server_random (gnutls_session_t session);
   const void *gnutls_session_get_client_random (gnutls_session_t session);
   const void *gnutls_session_get_master_secret (gnutls_session_t session);
 
+  typedef void
+  (*gnutls_finished_callback_func) (gnutls_session_t session,
+				    const void *finished,
+				    size_t len);
+  void
+  gnutls_session_set_finished_function (gnutls_session_t session,
+					gnutls_finished_callback_func
+					finished_func);
+
 /* checks if this session is a resumed one 
  */
   int gnutls_session_is_resumed (gnutls_session_t session);
@@ -537,6 +646,10 @@ extern "C"
   int gnutls_db_check_entry (gnutls_session_t session,
 			     gnutls_datum_t session_entry);
 
+  typedef int (*gnutls_handshake_post_client_hello_func)(gnutls_session_t);
+  void gnutls_handshake_set_post_client_hello_function(gnutls_session_t,
+      gnutls_handshake_post_client_hello_func);
+  
   void gnutls_handshake_set_max_packet_length (gnutls_session_t session,
 					       size_t max);
 
@@ -635,10 +748,12 @@ extern "C"
 					   const gnutls_datum_t * KEY,
 					   gnutls_x509_crt_fmt_t type);
 
-  extern int
-    gnutls_certificate_set_x509_simple_pkcs12_file
-    (gnutls_certificate_credentials_t res, const char *pkcs12file,
-     gnutls_x509_crt_fmt_t type, const char *password);
+  void gnutls_certificate_send_x509_rdn_sequence (gnutls_session_t session,
+						  int status);
+
+  int gnutls_certificate_set_x509_simple_pkcs12_file
+  (gnutls_certificate_credentials_t res, const char *pkcs12file,
+   gnutls_x509_crt_fmt_t type, const char *password);
 
 /* New functions to allow setting already parsed X.509 stuff.
  */
@@ -651,6 +766,9 @@ extern "C"
   struct gnutls_x509_crt_int;
   typedef struct gnutls_x509_crt_int *gnutls_x509_crt_t;
 
+  struct gnutls_openpgp_keyring_int;
+  typedef struct gnutls_openpgp_keyring_int *gnutls_openpgp_keyring_t;
+
   int gnutls_certificate_set_x509_key (gnutls_certificate_credentials_t res,
 				       gnutls_x509_crt_t * cert_list,
 				       int cert_list_size,
@@ -662,6 +780,17 @@ extern "C"
 				       gnutls_x509_crl_t * crl_list,
 				       int crl_list_size);
 
+  void gnutls_certificate_get_x509_cas (gnutls_certificate_credentials_t sc,
+					gnutls_x509_crt_t **x509_ca_list,
+					unsigned int* ncas);
+
+  void gnutls_certificate_get_x509_crls (gnutls_certificate_credentials_t sc,
+					 gnutls_x509_crl_t **x509_crl_list,
+					 unsigned int* ncrls);
+
+  void gnutls_certificate_get_openpgp_keyring (gnutls_certificate_credentials_t sc,
+					       gnutls_openpgp_keyring_t *keyring);
+
 /* global state functions
  */
   int gnutls_global_init (void);
@@ -673,14 +802,13 @@ extern "C"
   typedef void (*gnutls_free_function) (void *);
   typedef void *(*gnutls_realloc_function) (void *, size_t);
 
-  extern void
-    gnutls_global_set_mem_functions (gnutls_alloc_function gt_alloc_func,
-				     gnutls_alloc_function
-				     gt_secure_alloc_func,
-				     gnutls_is_secure_function
-				     gt_is_secure_func,
-				     gnutls_realloc_function gt_realloc_func,
-				     gnutls_free_function gt_free_func);
+  void gnutls_global_set_mem_functions (gnutls_alloc_function gt_alloc_func,
+					gnutls_alloc_function
+					gt_secure_alloc_func,
+					gnutls_is_secure_function
+					gt_is_secure_func,
+					gnutls_realloc_function gt_realloc_func,
+					gnutls_free_function gt_free_func);
 
 /* For use in callbacks */
   extern gnutls_alloc_function gnutls_malloc;
@@ -779,8 +907,8 @@ extern "C"
   void gnutls_session_set_ptr (gnutls_session_t session, void *ptr);
   void *gnutls_session_get_ptr (gnutls_session_t session);
 
-  void gnutls_openpgp_send_key (gnutls_session_t session,
-				gnutls_openpgp_key_status_t status);
+  void gnutls_openpgp_send_cert (gnutls_session_t session,
+				gnutls_openpgp_crt_status_t status);
 
 /* fingerprint 
  * Actually this function returns the hash of the given data.
@@ -815,12 +943,15 @@ extern "C"
 
   const char *gnutls_srp_server_get_username (gnutls_session_t session);
 
-  extern int gnutls_srp_verifier (const char *username,
-				  const char *password,
-				  const gnutls_datum_t * salt,
-				  const gnutls_datum_t * generator,
-				  const gnutls_datum_t * prime,
-				  gnutls_datum_t * res);
+  extern void gnutls_srp_set_prime_bits (gnutls_session_t session,
+					 unsigned int bits);
+
+  int gnutls_srp_verifier (const char *username,
+			   const char *password,
+			   const gnutls_datum_t * salt,
+			   const gnutls_datum_t * generator,
+			   const gnutls_datum_t * prime,
+			   gnutls_datum_t * res);
 
 /* The static parameters defined in draft-ietf-tls-srp-05
  * Those should be used as input to gnutls_srp_verifier().
@@ -848,8 +979,7 @@ extern "C"
      gnutls_srp_server_credentials_function * func);
 
   typedef int gnutls_srp_client_credentials_function (gnutls_session_t,
-						      unsigned int, char **,
-						      char **);
+						      char **, char **);
   void
     gnutls_srp_set_client_credentials_function
     (gnutls_srp_client_credentials_t cred,
@@ -893,7 +1023,12 @@ extern "C"
   int gnutls_psk_set_server_credentials_file (gnutls_psk_server_credentials_t
 					      res, const char *password_file);
 
+  int
+  gnutls_psk_set_server_credentials_hint (gnutls_psk_server_credentials_t res,
+					  const char *hint);
+
   const char *gnutls_psk_server_get_username (gnutls_session_t session);
+  const char *gnutls_psk_client_get_hint (gnutls_session_t session);
 
   typedef int gnutls_psk_server_credentials_function (gnutls_session_t,
 						      const char *username,
@@ -923,6 +1058,12 @@ extern "C"
 					      res,
 					      gnutls_params_function * func);
 
+  int gnutls_psk_netconf_derive_key (const char *password,
+				     const char *psk_identity,
+				     const char *psk_identity_hint,
+				     gnutls_datum_t *output_key);
+
+
   typedef enum gnutls_x509_subject_alt_name_t
   {
     GNUTLS_SAN_DNSNAME = 1,
@@ -937,8 +1078,8 @@ extern "C"
     GNUTLS_SAN_OTHERNAME_XMPP = 1000
   } gnutls_x509_subject_alt_name_t;
 
-  struct gnutls_openpgp_key_int;
-  typedef struct gnutls_openpgp_key_int *gnutls_openpgp_key_t;
+  struct gnutls_openpgp_crt_int;
+  typedef struct gnutls_openpgp_crt_int *gnutls_openpgp_crt_t;
 
   struct gnutls_openpgp_privkey_int;
   typedef struct gnutls_openpgp_privkey_int *gnutls_openpgp_privkey_t;
@@ -949,7 +1090,7 @@ extern "C"
     union cert
     {
       gnutls_x509_crt_t *x509;
-      gnutls_openpgp_key_t pgp;
+      gnutls_openpgp_crt_t pgp;
     } cert;
     unsigned int ncerts;	/* one for pgp keys */
 
@@ -1089,6 +1230,9 @@ extern "C"
   void gnutls_psk_set_params_function (gnutls_psk_server_credentials_t res,
 				       gnutls_params_function * func);
 
+  int gnutls_hex2bin (const char * hex_data, size_t hex_size,
+		      char * bin_data, size_t * bin_size);
+
   /* Gnutls error codes. The mapping to a TLS alert is also shown in
    * comments.
    */
@@ -1188,7 +1332,6 @@ extern "C"
 #define GNUTLS_E_ASN1_TYPE_ANY_ERROR -75
 #define GNUTLS_E_ASN1_SYNTAX_ERROR -76
 #define GNUTLS_E_ASN1_DER_OVERFLOW -77
-#define GNUTLS_E_OPENPGP_TRUSTDB_VERSION_UNSUPPORTED -81
 #define GNUTLS_E_OPENPGP_UID_REVOKED -79
 #define GNUTLS_E_CERTIFICATE_ERROR -43
 #define GNUTLS_E_X509_CERTIFICATE_ERROR GNUTLS_E_CERTIFICATE_ERROR
@@ -1209,6 +1352,8 @@ extern "C"
 
 #define GNUTLS_E_IA_VERIFY_FAILED -104
 
+#define GNUTLS_E_UNKNOWN_ALGORITHM -105
+
 #define GNUTLS_E_BASE64_ENCODING_ERROR -201
 #define GNUTLS_E_INCOMPATIBLE_GCRYPT_LIBRARY -202	/* obsolete */
 #define GNUTLS_E_INCOMPATIBLE_CRYPTO_LIBRARY -202
@@ -1218,15 +1363,19 @@ extern "C"
 #define GNUTLS_E_X509_UNSUPPORTED_OID -205
 
 #define GNUTLS_E_RANDOM_FAILED -206
+#define GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR -207
+
+#define GNUTLS_E_OPENPGP_SUBKEY_ERROR -208
+
+#define GNUTLS_E_CRYPTO_ALREADY_REGISTERED -209
+
+#define GNUTLS_E_HANDSHAKE_TOO_LARGE -210
 
 #define GNUTLS_E_UNIMPLEMENTED_FEATURE -1250
 
 #define GNUTLS_E_APPLICATION_ERROR_MAX -65000
 #define GNUTLS_E_APPLICATION_ERROR_MIN -65500
 
-  /* Backwards compatibility mappings. */
-#include <gnutls/compat.h>
-
 #ifdef __cplusplus
 }
 #endif
